#!/usr/bin/env python3.6

'''
for main function and description see main_create_GNVN.py

This work is licensed under the Creative Commons Attribution-ShareAlike 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/4.0/.

Website: http://www.cl.uni-heidelberg.de/projects/GNVN_semanno/
Contact: Eva Mujdricza-Maydt, mujdricza@cl.uni-heidelberg.de
'''

__author__ = "mujdricza-maydt"
__contact__ = "mujdricza@cl.uni-heidelberg.de"
__version__ = "20170810"

'''
utility methods and constants (enum) for handling data and data
'''

from enum import Enum
import os


class FORMAT(Enum):
    '''commonly used variables'''
    TAB = "\t"
    NL = "\n"
    COMMENTLINE_start = "#"


class DATASET(Enum):
    '''dataset names'''
    train = "train"
    dev = "dev"
    test = "test"


class CoNLL2009_IDX(Enum):
    FILLPRED = 12


class CoNLL2009_FILES(Enum):
    '''assumed names for the CoNLL2009 data files'''
    train = "CoNLL2009-ST-German-train.txt"
    dev = "CoNLL2009-ST-German-development.txt"
    test = "CoNLL2009-ST-evaluation-German.txt"


class GNVN_CoNLL2009onlySRL_FILES(Enum):
    '''names for the srl-only gnvn files from the CoNLL2009 data '''
    train = "CoNLL2009-ST-German-train.txt_onlySRL.conll"
    dev = "CoNLL2009-ST-German-development.txt_onlySRL.conll"
    test = "CoNLL2009-ST-evaluation-German.txt_onlySRL.conll"


class GNVN_CoNLL2009_FILES(Enum):
    '''names for the output files (CoNLL2009 data) '''
    train = "GNVN_CoNLL2009_train.conll"
    dev = "GNVN_CoNLL2009_dev.conll"
    test = "GNVN_CoNLL2009_test.conll"


def create_path_if_needed(output_path):
    os.makedirs(output_path, exist_ok=True)


def normalize_path(path_name):
    if not path_name.endswith(os.path.sep):
        path_name =+ os.path.sep
    return path_name


def create_path_structure(shared_path, collection_of_subpaths):
    for subpath in collection_of_subpaths:
        create_path_if_needed(shared_path + subpath)


def read_tabulated(input_file_name, minimum_columns=2):
    list_of_sentences = []
    
    current_sentence = []
    with open(input_file_name) as fin:
        for line in fin.readlines():
            
            if __is_not_sentence_line(line):
                if current_sentence:
                    list_of_sentences.append(current_sentence)
                    current_sentence = []
            else:
                current_sentence.append(line.split())
    
    if current_sentence:
        list_of_sentences.append(current_sentence)
    
    return list_of_sentences


def write_tabulated(output_file_name, list_of_sentences, column_separator=FORMAT.TAB.value):
    
    with open(output_file_name, "w") as fout:
        for sentence in list_of_sentences:
            for line_split in sentence:
                fout.write(column_separator.join(line_split))
                fout.write(FORMAT.NL.value)
            fout.write(FORMAT.NL.value)
    

def __is_not_sentence_line(line):
    '''empty or commented lines are not relevant'''
    line = line.strip()
    if len(line) == 0:
        return True
    if line.startswith(FORMAT.COMMENTLINE_start.value):
        return True
    return False


def get_fully_annotated_sentence_list(gnvn_onlySRL_sentence_list, conll2009_sentence_list):
    __check_list_lengths(gnvn_onlySRL_sentence_list, conll2009_sentence_list)
    
    fully_annotated_sentence_list = []
    for i in range(len(gnvn_onlySRL_sentence_list)):
        gnvn_sentence = gnvn_onlySRL_sentence_list[i]
        conll2009_sentence = conll2009_sentence_list[i]
        
        fully_annotated_sentence = __merge_annobase_with_onlySRL(conll2009_sentence, gnvn_sentence)
        fully_annotated_sentence_list.append(fully_annotated_sentence)
    
    return fully_annotated_sentence_list


def __merge_annobase_with_onlySRL(conll2009_sentence, only_srl_sentence):
    fully_annotated_sentence = []
    conll2009_sentence_without_srl = __cut_off_srl_annotation(conll2009_sentence)
    __check_list_lengths(only_srl_sentence, conll2009_sentence_without_srl)
    
    for i in range(len(conll2009_sentence_without_srl)):
        line_base = conll2009_sentence_without_srl[i]
        line_srl = only_srl_sentence[i]
        line = line_base + line_srl
        fully_annotated_sentence.append(line)
    
    return fully_annotated_sentence

def __cut_off_srl_annotation(conll_formatted_sentence):
    conll_base_sentence = []
    
    for conll_formatted_line in conll_formatted_sentence:
        conll_base_line = conll_formatted_line[:CoNLL2009_IDX.FILLPRED.value]
        conll_base_sentence.append(conll_base_line)
    
    return conll_base_sentence

def __check_list_lengths(list_1, list_2):
    '''for lists which should have the same length'''
    if len(list_1) != len(list_2):
        raise IndexError("NOT EXPECTED UNEQUALITY IN LIST LENGTHS list_1:{} != list_2:{}\nLIST_1: {}\nLIST_2: {}".format(len(list_1), len(list_2), list_1, list_2))